Scriptname LoversComfortSpouseScr extends ReferenceAlias  

LoversComfortUtilScr Property lcUtil Auto
LoversComfortPlayerScr Property lcPlayer Auto
LoversComfortConfigScr Property lcConfig Auto

Quest Property lcSpouseNoteQuest Auto
Quest Property lcLoverNoteQuest Auto
Quest Property lcLoverHello04Quest Auto
Quest Property lcLoverHello02Quest Auto

ReferenceAlias Property lcNoteFromSpouse Auto
ReferenceAlias Property lcNoteFromLover Auto

Actor Property PlayerRef Auto
WiCourierScript Property WICourierScr Auto
GlobalVariable Property GameDaysPassed Auto

Float LastSpouseNoteTime = 0.0;
Float NextSceneTime03 = 0.0;
Float NextSceneTime04 = 0.0;

Event OnUpdate()
	Actor akSpouse = GetActorReference()
	
	If (akSpouse == None)
		; no spouse shut-down script
		Return
	EndIf
	
	Actor akLover = lcPlayer.GetSpouseLover()
	
	Actor tmpAk = Game.FindRandomActorFromRef(akSpouse, 1024)
	If (lcUtil.IsValidActor(tmpAk) >= 0)
		akLover = SelectLover(tmpAk, akLover)
		lcPlayer.SetSpouseLover(akLover)
	EndIf
	
	UpdateLoverRelationship(akSpouse, akLover)
	SpouseSendCourier(akSpouse)
	
	RegisterForSingleUpdate(15.0)
EndEvent

Function DoLoversScan(Actor[] validActorList)
	Actor akLover = lcPlayer.GetSpouseLover()

	int i = 0
	While (i < validActorList.length)
		Actor tmpAk = validActorList[i]
		akLover = SelectLover(tmpAk, akLover)
		i += 1
	EndWhile
	
	lcPlayer.SetSpouseLover(akLover)
	
	RegisterForSingleUpdate(1.0)
EndFunction


Actor Function SelectLover(Actor akRef, Actor prevLover)
	Actor akSpouse = GetActorReference()
	
	If (akRef == None)
		Return prevLover
	ElseIf (akSpouse == None)
		Return prevLover
	ElseIf (akSpouse == akRef)
		Return prevLover
	EndIf
	
;	Debug.Trace(Self + ": SelectLover trying to match " + akRef.GetLeveledActorBase().GetName())
	
	If (prevLover != None)
		If (akSpouse == prevLover)
			prevLover = None
		ElseIf (!lcUtil.IsInSameSettlement(akSpouse, prevLover))
			prevLover = None
		ElseIf (!lcConfig.IsSpouseGay && akSpouse.GetLeveledActorBase().GetSex() == prevLover.GetLeveledActorBase().GetSex())
			prevLover = None
		EndIf
	EndIf
	
	; check settings
	If (!(!lcConfig.IsSpouseGay && akSpouse.GetLeveledActorBase().GetSex() == akRef.GetLeveledActorBase().GetSex()))
		If (prevLover == None)
			If (akRef.GetRelationshipRank(akSpouse) >= 0)
				Return akRef
			EndIf
		ElseIf (akRef.GetRelationshipRank(akSpouse) > prevLover.GetRelationshipRank(akSpouse))
			Return akRef
		ElseIf (lcUtil.GetSpouseLoverRank(akRef) > lcUtil.GetSpouseLoverRank(prevLover))
			Return akRef
		EndIf
	EndIf
	
	Return prevLover
EndFunction


Function UpdateLoverRelationship(Actor akSpouse, Actor akSpouseLover)
	If (akSpouse == None || akSpouseLover == None)
		return
	ElseIf (!lcUtil.IsInSameSettlement(akSpouse, akSpouseLover))
		return
	EndIf
	
	Int loverRank = lcUtil.GetSpouseLoverRank(akSpouseLover)
	Int hoursSinceSex = lcUtil.slaUtil.GetActorHoursSinceLastSex(akSpouseLover)
	
	If (lcUtil.slaUtil.GetActorArousal(akSpouse) > lcConfig.SpouseArousalThreshold &&  (hoursSinceSex >= 20 || hoursSinceSex < 0))
		; cheap cooldown method
		lcUtil.slaUtil.UpdateActorLastSexTime(akSpouseLover)
		loverRank = lcUtil.UpdateSpouseLoverRank(akSpouseLover, 1)
		lcUtil.slaUtil.UpdateActorExposure(akSpouse, -10, "contact with lover " + akSpouseLover.GetLeveledActorBase().GetName())

		If (loverRank == 3)
			If (!lcLoverNoteQuest.IsRunning())
				lcLoverNoteQuest.Start()
			Else
				lcNoteFromLover.GetRef().Delete()
				lcLoverNoteQuest.Stop()
				lcLoverNoteQuest.Start()
			EndIf
		EndIf	
	EndIf
	
	If (akSpouse.GetCurrentLocation() != PlayerRef.GetCurrentLocation())
;		Debug.Trace(Self + ": Player and Spouse are in different locations")
		Return
	ElseIf (akSpouse.GetCurrentLocation() != akSpouseLover.GetCurrentLocation())
		Return
	ElseIf (akSpouse.IsInFaction(lcPlayer.SexLab.AnimatingFaction))
		Debug.Trace(Self + ": Spouse busy having sex")
		Return
	ElseIf (akSpouseLover.IsInFaction(lcPlayer.SexLab.AnimatingFaction))
		Debug.Trace(Self + ": Lover busy having sex")
		Return
	EndIf	
		
	If (loverRank >= 5 && lcUtil.slaUtil.GetActorArousal(akSpouse) > lcConfig.SpouseArousalThreshold)
		Actor[] tmpList = new Actor[2]
		tmpList[0] = akSpouse
		tmpList[1] = akSpouseLover
		lcPlayer.MakeLove(tmpList)
		Return
	EndIf
	
	If (lcLoverHello02Quest.IsRunning())
		Debug.Trace(Self + ": Lover Hello 02 Quest is running")
		Return
	ElseIf (lcLoverHello04Quest.IsRunning())
		Debug.Trace(Self + ": Lover Hello 04 Quest is running")
		Return
	EndIf
		
	If (loverRank == 2 && lcConfig.IsSpouseScenesEnabled && NextSceneTime03 < GameDaysPassed.GetValue())
		Debug.Trace(Self + ": Started Lover Hello 02 Quest...")
		lcLoverHello02Quest.Start()
		NextSceneTime03 = GameDaysPassed.GetValue() + 1.0
		Return
	EndIf
		
	If (loverRank == 4 && lcConfig.IsSpouseScenesEnabled && NextSceneTime04 < GameDaysPassed.GetValue())
		Debug.Trace(Self + ": Started Lover Hello 04 Quest...")
		lcLoverHello04Quest.Start()
		NextSceneTime04 = GameDaysPassed.GetValue() + 1.0
		Return
	EndIf
EndFunction


Function SpouseSendCourier(Actor akSpouse)
	If (akSpouse == None)
		Return
	EndIf
	
	If (!lcSpouseNoteQuest.IsRunning())
		lcSpouseNoteQuest.Start()
	EndIf
		
	If (lcSpouseNoteQuest.GetStage() == 0 && lcUtil.slaUtil.GetActorArousal(akSpouse) > lcConfig.SpouseArousalThreshold && PlayerRef.GetItemCount(lcNoteFromSpouse.GetReference().GetBaseObject()) < 1 && !lcUtil.IsCurrentFollower(akSpouse))
		; send message
		WICourierScr.addAliasToContainer(lcNoteFromSpouse)
		LastSpouseNoteTime = GameDaysPassed.GetValue()
		lcSpouseNoteQuest.SetStage(10)
	ElseIf (lcSpouseNoteQuest.GetStage() == 10 && WICourierScr.pCourierContainer.GetItemCount(lcNoteFromSpouse.GetReference().GetBaseObject()) < 1)
		; curier don't have message
		lcSpouseNoteQuest.SetStage(20)
	ElseIf (lcSpouseNoteQuest.GetStage() == 20 && GameDaysPassed.GetValue() - LastSpouseNoteTime > 7.0)
		; delete note and reset quest
		lcNoteFromSpouse.GetRef().Delete()
		lcSpouseNoteQuest.Stop()
	EndIf
EndFunction